@@ -2,14 +2,15 @@
 use strict;
 use warnings;
 
-package filename v0.20.008;
+package filename v0.20.009;
 # ABSTRACT: Perl module to load files at compile-time, without BEGIN blocks.
 
 
-use Carp 1.50 ();
+use Carp 1.50  ();
+use File::Spec ();
 
 
-my $error = sub { };    # Private sub
+my ( $do, $error ) = ();    # Private subs
 
 # Modified version of the code as specified in `perldoc -f require`
 *import = \&require;
@@ -21,27 +22,46 @@ sub require {
 
     return $INC{$filename} if ( exists $INC{$filename} );
 
-    if ( $filename =~ m!\A/! ) {
+    if ( File::Spec->file_name_is_absolute($filename) ) {
         goto NOT_INC if $^V < v5.17.0 && !-r $filename;
-        return do($filename) || $error->($filename);
+        return $do->($filename);
     }
-    foreach my $prefix (@INC) {
+    foreach my $inc (@INC) {
+        my $prefix = $inc;
+        #if ( my $ref = ref($inc) ) {
+        #    # TODO ...
+        #}
         next unless -f ( my $fullpath = "$prefix/$filename" );
         next if $^V < v5.17.0 && !-r _;
-        my $result = do $fullpath;
-        $INC{$filename} = delete $INC{$fullpath};
-        return $result || $error->( $filename => $fullpath );
+        return $do->( $fullpath => $filename );
     }
     NOT_INC:
         Carp::croak("Can't locate $filename in \@INC (\@INC contains: @INC)");
 }
 
+my $do_text = <<'END';
+package $pkg;
+my $result = do($fullpath);
+die $@ if $@;
+$INC{$filename} = delete $INC{$fullpath}
+    if $filename ne $fullpath && exists $INC{$fullpath};
+$result;
+END
+$do = sub {
+    my $fullpath = @_ ? shift : $_;
+    my $filename = @_ ? shift : $fullpath;
+    my ($pkg)    = caller(2);
+
+    ( my $do_eval = $do_text ) =~ s/\$pkg/$pkg/;
+    return eval $do_eval || $error->( $filename => $fullpath );
+};
+
 # Private sub
 $error = sub {
     my $filename = @_ ? shift : $_;
     my $fullpath = @_ ? shift : $filename;
 
-    $INC{$filename} = undef;
+    $INC{$filename} &&= undef($!);
 
     $@ && Carp::croak( $@, "Compilation failed in require" );
 
@@ -51,7 +71,6 @@ $error = sub {
         "$!"
     );
 
-    delete $INC{$filename};
     Carp::croak( $filename, " did not return a true value" );
 };
 
@@ -113,9 +132,17 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 TODO
+
+=over
+
+=item * Handle references in C<@INC>
+
+=back
+
 =head1 VERSION
 
-This document describes version v0.20.008 of this module.
+This document describes version v0.20.009 of this module.
 
 =head1 AUTHOR
 
